using System;
using System.Drawing;
using System.Windows.Forms;
using Metrix.Contracts;
using Plux;
using Plux.Framework;

[Extension("PieChart")]
[Plug("Metrix.Control")]
[Slot("Plux.Figure", Multiple = true, LazyLoad=true,
  OnPlugged = "Figure_Plugged", OnUnplugged = "Figure_Unplugged")]
[ParamValue("Caption", "Pie")]
[ParamValue("OrderIndex", 0.5f)]
[ReplaceSlot("Plux.Figure", Plug = true)]
class Pie : Control, ITabPage {
  public Pie() {
    Padding = new Padding(12);
    BackColor = Color.Snow;
  }
  #region ITabPage Members
  public Control Control {
    get { return this; }
  }
  #endregion
  protected override void OnPaint(PaintEventArgs args) {
    base.OnPaint(args);
    SlotInfo slot = Runtime.GetSlotInfo(this, "Plux.Figure");
    Rectangle rc = ClientRectangle;
    if(slot.PluggedPlugInfos.Count == 0) {
      TextRenderer.DrawText(args.Graphics, "No measures", Font, rc,
                            SystemColors.WindowText,
                            TextFormatFlags.HorizontalCenter
                            | TextFormatFlags.VerticalCenter);
      return;
    }

    IFigure f = (IFigure)slot.PluggedPlugInfos[0].Object;
    int sum = 0;
    foreach(IMeasurement m in f.Measurements) {
      sum += (int) m.Current;
    }

    rc.Inflate(-Padding.Right, -Padding.Bottom);
    Color[] colors = {
      Color.Orange, Color.LightSteelBlue, Color.YellowGreen,
      Color.Yellow, Color.Red, Color.Silver
    };

    int cnt = 0;
    int pie = 0;
    foreach(IMeasurement m in f.Measurements) {
      int cur = (int) m.Current;
      if(cur > 0) {
        using(Brush b = new SolidBrush(colors[cnt++%colors.Length])) {
          int angle = cur * 360 / sum;
          args.Graphics.FillPie(b, rc, pie, angle);
          pie += angle;
        }
      }
    }
  }
  public void Figure_Plugged(object s, PlugEventArgs args) {
    Invalidate();
  }
  public void Figure_Unplugged(object s, PlugEventArgs args) {
    Invalidate();
  }
}