using System.Collections.Generic;

namespace Plux.Framework {
  public static class SlotInfoEx {
    public static ExtensionInfo FindUniqueExtension
      (SlotInfo slot, ExtensionTypeInfo extType) {
      foreach(ExtensionInfo ext in extType.ExtensionInfos)
        if(ext.Owner == slot) return ext;
      return null;
    }
    public static PlugTypeInfo FindRegisteredPlug(SlotInfo slot, string extName) {
      foreach (PlugTypeInfo plugType in slot.RegisteredPlugTypeInfos) {
        if (!plugType.ExtensionTypeInfo.Name.Equals(extName)) continue;
        return plugType;
      }
      return null;
    }
    public static PlugTypeInfo FindRegisteredPlug(SlotInfo slot, string param, string value) {
      foreach(PlugTypeInfo plugType in slot.RegisteredPlugTypeInfos) {
        if(!plugType.GetParamValue(param).Equals(value)) continue;
        return plugType;
      }
      return null;
    }
    public static PlugInfo FindPluggedPlug(SlotInfo slot, string param, string value) {
      foreach (PlugInfo plug in slot.PluggedPlugInfos) {
        if (!plug.PlugTypeInfo.GetParamValue(param).Equals(value)) continue;
        return plug;
      }
      return null;
    }
    public static PlugInfo FindPluggedPlug(SlotInfo slot, string extName) {
      foreach(PlugInfo plug in slot.PluggedPlugInfos) {
        if(!plug.PlugTypeInfo.ExtensionTypeInfo.Name.Equals(extName)) continue;
        return plug;
      }
      return null;
    }
    public static void CreateAndPlug(SlotInfo slot) {
      if (!slot.LazyLoad && slot.AutoPlug) return;
      if (slot.RegisteredPlugTypeInfos.Count == 0) return;
      List<PlugTypeInfo> pluggedTypes = new List<PlugTypeInfo>();
      foreach (PlugInfo plug in slot.PluggedPlugInfos) {
        if (pluggedTypes.Contains(plug.PlugTypeInfo)) continue;
        pluggedTypes.Add(plug.PlugTypeInfo);
      }
      foreach (PlugTypeInfo plugType in slot.RegisteredPlugTypeInfos) {
        if (pluggedTypes.Contains(plugType)) continue;
        PlugInfo plug = plugType.CreateExtension(slot);
        if (!slot.AutoPlug || slot.Unique)
          plug.Plug(slot);
      }
    }
    public static PlugInfo GetPluggedPlug(this SlotInfo slot, PlugTypeInfo plugType) {
      foreach (PlugInfo plug in slot.PluggedPlugInfos) {
        if (plug.PlugTypeInfo != plugType) continue;
        return plug;
      }
      return null;
    }
    public static bool IsPlugged(this SlotInfo slot, PlugTypeInfo plugType) {
      return (GetPluggedPlug(slot, plugType) != null);
    }
  }
}